@@ -1,10 +1,12 @@
 use utf8;
 package TaskPipe::SchemaTemplate_Global::Result::Thread;
+
 use strict;
 use warnings;
 
 use base 'DBIx::Class::Core';
 
+__PACKAGE__->load_components("InflateColumn::DateTime");
 
 __PACKAGE__->table("thread");
 
@@ -22,10 +24,29 @@ __PACKAGE__->add_columns(
   { data_type => "varchar", is_nullable => 1, size => 255 },
   "status",
   { data_type => "varchar", is_nullable => 1, size => 255 },
+  "token",
+  { data_type => "varchar", is_nullable => 1, size => 255 },
+  "data",
+  { data_type => "mediumtext", is_nullable => 1 },
+  "last_forked",
+  {
+    data_type => "datetime",
+    datetime_undef_if_invalid => 1,
+    is_nullable => 1,
+  },
+  "last_checked",
+  {
+    data_type => "datetime",
+    datetime_undef_if_invalid => 1,
+    is_nullable => 1,
+  },
+  "last_task",
+  { data_type => "varchar", is_nullable => 1, size => 255 },
 );
 
 __PACKAGE__->set_primary_key("id", "job_id");
 
+
 =head1 NAME
 
 TaskPipe::SchemaTemplate_Global::Result::Thread - global schema template for C<thread> source
@@ -47,4 +68,5 @@ TaskPipe is free software, licensed under
     The GNU Public License Version 3
 
 =cut
+
 1;
