@@ -27,7 +27,7 @@ has template => (is => 'ro', isa => 'Str', default => q|---
 
 
 -   _name: Record
-    table: Company
+    table: company
     values:
         ticker: $this[1]
         url: $this[1]
@@ -40,4 +40,29 @@ has template => (is => 'ro', isa => 'Str', default => q|---
         quote: $this
 |);
 
+
+=head1 NAME
+
+TaskPipe::Template_Plan_SP500 - template for the SP500 sample project plan
+
+=head1 DESCRIPTION
+
+This is the template which creates the plan for the SP500 sample project
+
+=head1 AUTHOR
+
+Tom Gracey <tomgracey@gmail.com>
+
+=head1 COPYRIGHT AND LICENSE
+
+Copyright (c) Tom Gracey 2018
+
+TaskPipe is free software, licensed under
+
+    The GNU Public License Version 3
+
+=cut
+
+
+
 1;
